@@ -1,16 +1,17 @@
 /*
- * Copyright 2005-2007 Gentoo Foundation
+ * Copyright 2005-2008 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.147 2007/05/30 23:17:24 solar Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/main.c,v 1.153 2008/01/16 16:24:49 solar Exp $
  *
- * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2008 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2008 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #define _GNU_SOURCE
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <sys/types.h>
 #include <string.h>
 #include <unistd.h>
@@ -249,6 +250,28 @@ static char *remove_extra_space(char *str)
 	return str;
 }
 
+static char *pkg_name(const char *const_name);
+static char *pkg_name(const char *const_name) {
+	static char name[_POSIX_PATH_MAX];
+	char *ptr;
+	if (!const_name)
+		return NULL;
+	strncpy(name, const_name, sizeof(name));
+	if ((ptr = strrchr(name, ':')) != NULL)
+		*ptr = 0;
+	return name;
+}
+
+static char *slot_name(const char *name);
+static char *slot_name(const char *name) {
+	char *ptr;
+	if (!name)
+		return NULL;
+	if ((ptr = strrchr(name, ':')) != NULL)
+		return ptr+1;
+	return NULL;
+}
+
 void freeargv(int, char **);
 void freeargv(int argc, char **argv)
 {
@@ -281,7 +304,7 @@ void makeargv(char *string, int *argc, char ***argv)
 
 		if (*argc == curc) {
 			curc *= 2;
-			(*argv) = (char **) realloc(*argv, sizeof(char **) * curc);
+			(*argv) = xrealloc(*argv, sizeof(char **) * curc);
 		}
 		(*argv)[*argc] = xstrdup(str);
 		(*argc)++;
@@ -474,7 +497,7 @@ void initialize_portage_env(void)
 				rmspace(buf);
 				if (*buf == '#' || *buf == '\0')
 					continue;
-				for (i=0; i<ARR_SIZE(vars_to_read); ++i) {
+				for (i=0; i < ARR_SIZE(vars_to_read); ++i) {
 					if (buf[vars_to_read[i].name_len] != '=' && buf[vars_to_read[i].name_len] != ' ')
 						continue;
 					if (strncmp(buf, vars_to_read[i].name, vars_to_read[i].name_len))
@@ -523,7 +546,7 @@ void initialize_portage_env(void)
 	} while (1);
 
 	/* finally, check the env */
-	for (i=0; i<ARR_SIZE(vars_to_read); ++i) {
+	for (i=0; i < ARR_SIZE(vars_to_read); ++i) {
 		s = getenv(vars_to_read[i].name);
 		if (s != NULL) {
 			switch (vars_to_read[i].type) {
@@ -685,7 +708,7 @@ const char *initialize_flat(int cache_type)
 	if (frac < 0) frac = 0;
 
 	warn("Finished %u entries in %d.%06d seconds", count, secs, frac);
-	if (secs > 100)
+	if (secs > 120)
 #ifdef __linux__
 		warn("You should consider a faster file system such as reiserfs for PORTDIR='%s'", portdir);
 #else
@@ -908,6 +931,7 @@ queue *get_vdb_atoms(int fullcpv)
 	int dfd, i;
 
 	char buf[_Q_PATH_MAX];
+	char slot[_Q_PATH_MAX];
 	static char savecwd[_POSIX_PATH_MAX];
 
 	struct dirent **cat;
@@ -947,11 +971,14 @@ queue *get_vdb_atoms(int fullcpv)
 			if ((atom = atom_explode(buf)) == NULL)
 				continue;
 
+			slot[0] = '0';
+			slot[1] = 0;
 			strncat(buf, "/SLOT", sizeof(buf));
 			if (access(buf, R_OK) == 0) {
 				eat_file(buf, buf, sizeof(buf));
 				rmspace(buf);
-				if (strcmp(buf, "0") != 0);
+				if (strcmp(buf, "0") != 0)
+					strncpy(slot, buf, sizeof(slot));
 			}
 
 			if (fullcpv) {
@@ -962,9 +989,8 @@ queue *get_vdb_atoms(int fullcpv)
 			} else {
 				snprintf(buf, sizeof(buf), "%s/%s", atom->CATEGORY, atom->PN);
 			}
-
 			atom_implode(atom);
-			cpf = add_set(buf, "0", cpf);
+			cpf = add_set(buf, slot, cpf);
 		}
 		chdir("..");
 		while (dfd--) free(pf[dfd]);
@@ -985,14 +1011,11 @@ void cleanup()
 	reinitialize_as_needed();
 	free_sets(virtuals);
 	fclose(stderr);
-#ifndef STATIC
-	if (dlhandle != NULL)
-		dlclose(dlhandle);
-#endif
 }
 
 int main(int argc, char **argv)
 {
+	struct stat st;
 	IF_DEBUG(init_coredumps());
 	argv0 = argv[0];
 
@@ -1001,12 +1024,13 @@ int main(int argc, char **argv)
 	bindtextdomain(argv0, "/usr/share/locale");
 	textdomain(argv0);
 #endif
-
-#if 0
-	if (ttyname(1) == NULL)
-		no_colors();
+#if 1
+	if (fstat(fileno(stdout), &st) != (-1))
+		if (!isatty(fileno(stdout)))
+			if ((S_ISFIFO(st.st_mode)) == 0)
+				no_colors();
 #endif
-	if (getenv("TERM") == NULL)
+	if ((getenv("TERM") == NULL) || ((strcmp(getenv("TERM"), "dumb")) == 0))
 		no_colors();
 
 	initialize_portage_env();
